// BWImageUtilities -- Image-related utility function

// Copyright (C) 2005 Borkware, LLC
// We use the BSD License.  Check out http://borkware.com/license

#import "BWImageUtilities.h"

// given a path and a rotation factor (number of 90-degree rotations to make,
// < 0 for clockwise rotation, > 0 for counter-clockwise rotation, return
// a new image that's the original file, but rotated

NSImage *BWImage_rotatedImage (NSString *filePath, int rotationFactor)
{
    // get the base image.  We might not have to do any violence to it
    NSImage *image = [[NSImage alloc] initWithContentsOfFile: filePath];

    // the actual factor may be outside the bounds of 0 .. 3
    // (0 == no rotation, 1 == 90 CCW, etc)
    // so trim the value to 0 .. 3

    rotationFactor %= 4;

    if (rotationFactor < 0) {
        rotationFactor += 4;
    }

    // not a "natural orientation" image, so make a new one

    if (rotationFactor > 0) {

         NSSize rotatedSize = [image size];

         // if the picture is rotated 180 degrees, the dimensions are the
         // same, otherwise exchange the width and the height
        if (rotationFactor != 2) {
            rotatedSize.width = [image size].height;
            rotatedSize.height = [image size].width;
        }

        // the destination of the rotated bits
        NSImage *rotatedImage;
        rotatedImage = [[NSImage alloc] initWithSize: rotatedSize];

        // to do the actual rotation, I'm doing the cheesy thing and
        // making a transform, and drawing the old image through
        // that transform.  I get acceptable results on my Ti500, which
        // is about half the speed of the target system

        NSAffineTransform *transform = [NSAffineTransform transform];

        // first skootch the origin to the right place, since that's
        // where the rotation will be around
        switch (rotationFactor) {
        case 1:
            [transform translateXBy: rotatedSize.width 
                       yBy: 0.0];
            break;
        case 2:
            [transform translateXBy: rotatedSize.width 
                       yBy: rotatedSize.height];
            break;
        case 3:
            [transform translateXBy: 0.0  
                       yBy: rotatedSize.height];
            break;
        }

        // and then spin things around
        [transform rotateByDegrees: 90 * rotationFactor];

        // now draw into the image
        [rotatedImage lockFocus]; {
            [transform concat];  // mutate the coordinate system first

            // this is the chunk of the original image (the entire thing)
            // to draw from
            NSRect sourceRect;
            sourceRect.origin = NSZeroPoint;
            sourceRect.size = [image size];

            // actually do the drawing
            [image drawAtPoint: NSZeroPoint  
                   fromRect: sourceRect
                   operation: NSCompositeSourceOver
                   fraction: 1.0];
        } [rotatedImage unlockFocus];
        
        // don't need the original any more
        [image release];
        
        image = rotatedImage;
    }

    return ([image autorelease]);

} // BWImage_rotatedImage

